/* Create a new MIDI file */

OPTIONS "C_CALL LABELCHECK NOSOURCE"

/* Add the REXX GUI and RxMidi functions */
LIBRARY rexxgui, rxmidi

/* Trap errors in RxMidi functions. So, we don't have to check the return values
 * from MIDI REXX functions that set the ERROR condition, such as MIDISetEvent() */
MidiErr = "ERROR"
MidiHeading = 1

/* Print an introduction */
IF GuiSay('This script creates a new MIDI file. Proceed?', 'YES|INFO', 'New MIDI') \= 'YES' THEN SIGNAL out

/* Check if we have a MIDI file name. We get the filename from
 * the command line (ie, the user typed it after disasm.rex), or
 * if not supplied, we bring up the file dialog.
 */
IF ARG() < 1 THEN DO
	error = GuiFile('FromFilename', 'OVERWRITE|SAVE', 'Enter a name for the new MIDI file', 'MIDI files (*.mid, *.rmi) | *.mid;*.rmi | All files (*.*) | *.*')
	IF error \= "" THEN RETURN
END
ELSE FromFilename = ARG(1)

/* Make sure that name ends in .MID */
IF TRANSLATE(RIGHT(FromFilename, 4)) \== ".MID" THEN FromFilename = FromFilename || '.mid'

DO

	/* Create a new MIDI file in RAM with 96 PPQN resolution */
	MIDIOpenFile()

	/* All events will be on first MIDI channel */
	MIDIEvent.!Channel = 1

	/* First event will be at the first downbeat (ie, 1:1:0) */
	MIDIEvent.!Measure = 1
	MIDIEvent.!Beat = 1
	MIDIEvent.!Clock = 0

	/* Add a Seq # event at the end of the first track */
	MIDIEvent.!Type = "Seq "
	MIDIEvent.!Data1 = 10		/* Seq # = 10 */
	MIDISetEvent('INS|TIME|DATA', 1)	/* For the first event, we select the track # we want */

	/* Add a Track Name event at the end of the first track */
	MIDIEvent.!Type = "Trac"
	MIDIEvent.!Data1 = "Dummy"
	MIDISetEvent('INS|TIME|DATA')		/* All other events, we use the currently selected track */

	/* Add an Instrument event at the end of the first track */
	MIDIEvent.!Type = "Inst"
	MIDIEvent.!Data1 = "Piano"
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Text event at the end of the first track */
	MIDIEvent.!Type = "Text"
	MIDIEvent.!Data1 = "Here's text"
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Time Sig event at the end of the first track */
	MIDIEvent.!Type = "Time"
	MIDIEvent.!Data1 = 2	/* 2/2 Time */
	MIDIEvent.!Data2 = 2
	MIDIEvent.!Data3 = 24	/* MIDI-clocks\click */
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Key Sig event at the end of the first track */
	MIDIEvent.!Type = "Key "
	MIDIEvent.!Data3 = "G Major"
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Tempo event */
	MIDIEvent.!Type = "Temp"
	MIDIEvent.!Data1 = 600000  /* micros\quarter */
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Port # event */
	MIDIEvent.!Type = "Port"
	MIDIEvent.!Data1 = 0
	MIDISetEvent('INS|TIME|DATA')

	/* Second Beat of first measure */
	MIDIEvent.!Beat = 2

	/* Add an On Note event (on first MIDI channel) */
	MIDIEvent.!Type = "On N"
	MIDIEvent.!Data1 = MIDINoteNum("E3")	/* note number */
	MIDIEvent.!Data2 = 127					/* velocity */
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* Add a Program event */
	MIDIEvent.!Type = "Prog"
	MIDIEvent.!Data1 = 1	/* program number (referenced from 0) */
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* Add a Sysex event */
	MIDIEvent.!Type = "Syse"
	MIDIEvent.!Data1 = "F0 01 F7"X  /* Or we could do MIDIEvent.!Data1 = X2C("F001F7") */
	MIDISetEvent('INS|TIME|DATA')

	/* Add an On Note event */
	MIDIEvent.!Type = "On N"
	MIDIEvent.!Data1 = MIDINoteNum("B3")
	MIDIEvent.!Data2 = 127
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* First Beat of second measure */
	MIDIEvent.!Measure = 2
	MIDIEvent.!Beat = 1

	/* Add a MIDI Stop event */
	MIDIEvent.!Type = "Stop"
	MIDISetEvent('INS|TIME|DATA')

	/* Add an On Note event */
	MIDIEvent.!Type = "On N"
	MIDIEvent.!Data1 = MIDINoteNum("G#3")
	MIDIEvent.!Data2 = 127
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* Second Beat of second measure */
	MIDIEvent.!Beat = 2

	/* Add an (Off) Note event */
	MIDIEvent.!Type = "(Off"
	MIDIEvent.!Data1 = MIDINoteNum("e3")
	MIDIEvent.!Data2 = ''
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* Add a Tempo event */
	MIDIEvent.!Type = "Temp"
	MIDIEvent.!Data1 = 500000
	MIDISetEvent('INS|TIME|DATA')

	/* Add an (Off) Note event */
	MIDIEvent.!Type = "(Off"
	MIDIEvent.!Data1 = MIDINoteNum("g#3")
	/* MIDIEvent.!Data2 = '' */
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* First Beat of third measure */
	MIDIEvent.!Measure = 3
	MIDIEvent.!Beat = 1

	/* Add a First Packet event */
	MIDIEvent.!Type = "Firs"
	MIDIEvent.!Data1 = "F0 02"X
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Packet event */
	MIDIEvent.!Type = "Pack"
	MIDIEvent.!Data1 = "03"X
	MIDISetEvent('INS|TIME|DATA')

	/* Add a Last Packet event */
	MIDIEvent.!Type = "Last"
	MIDIEvent.!Data1 = "04 F7"X
	MIDISetEvent('INS|TIME|DATA')

	/* First Beat of fourth measure */
	MIDIEvent.!Measure = 4

	/* Add a Specific event */
	MIDIEvent.!Type = "Spec"
	MIDIEvent.!Data1 = "00 01 02 03"X
	MIDISetEvent('INS|TIME|DATA')

	/* Add an (Off) Note event */
	MIDIEvent.!Type = "(Off"
	MIDIEvent.!Data1 = MIDINoteNum("b3")
	/* MIDIEvent.!Data2 = '' */
	MIDISetEvent('INS|TIME|DATA|CHAN')

	/* Set time to 4:1:30 */
	MIDIEvent.!Clock = 30

	/* Add an End of Track event */
	MIDIEvent.!Type = "End "
	MIDISetEvent('INS|TIME')

	/* Save to a MIDI file */
	MIDISaveFile(FromFilename)

	CATCH ERROR
		CONDITION('M')
END

RETURN